Package: testConfigLoadAll
Title: What the Package Does (One Line, Title Case)
Version: 0.0.0.9000
Authors@R: 
    person("First", "Last", , "first.last@example.com", role = c("aut", "cre"),
           comment = c(ORCID = "YOUR-ORCID-ID"))
Description: What the package does (one paragraph).
License: `use_mit_license()`, `use_gpl3_license()` or friends to pick a
    license
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.0
Config/testthat/load-all: list(export_all = FALSE, helpers = FALSE)
Imports:
    rlang
Suggests: 
    testthat (>= 3.0.0)
Config/testthat/edition: 3
