# reporter basics works

    'reporters/tests.R:13:3' [failure] Failure:1. Expected `x` to be TRUE.
    'reporters/tests.R:17:8' [failure] Failure:2a. Expected FALSE to be TRUE.
    'reporters/tests.R:24:3' [error] Error:1. Error in `eval(code, test_env)`: stop
    'reporters/tests.R:30:8' [error] errors get tracebacks. Error in `h()`: !
    'reporters/tests.R:38:3' [skip] explicit skips are reported. Reason: skip
    'reporters/tests.R:41:1' [skip] empty tests are implicitly skipped. Reason: empty test
    'reporters/tests.R:47:5' [warning] warnings get backtraces. def
    'reporters/tests.R:45:1' [skip] warnings get backtraces. Reason: empty test

