# basic skips work as expected

    Reason: Skipping

---

    Reason: TRUE is TRUE

---

    Reason: FALSE is not TRUE

---

    Reason: empty test

# autogenerated message is always single line

    Reason: a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name || a_very_long_argument_name is not TRUE

# skip_if_not_installed() works as expected, absence and version

    Reason: {doesntexist} is not installed

---

    Reason: Installed testthat is version 3.0.0; but 9999.9999.999 is required

# skip_if_not_installed() works as expected, offline

    Reason: offline

# skip_on_ci() works as expected

    Reason: On CI

# skip_on_covr() works as expected

    Reason: On covr

# skip_on_bioc() works as expected

    Reason: On Bioconductor

# superseded CI skips still work

    Reason: On Travis

---

    Reason: On Appveyor

# skip_if_translated() works as expected

    Reason: "'%s' not found" is translated

# skip on os checks os names

    Code
      skip_on_os("amiga")
    Condition
      Error in `match.arg()`:
      ! 'arg' should be one of "windows", "mac", "linux", "solaris", "emscripten"

# can skip on multiple oses

    Reason: On Windows

---

    Reason: On Windows

# can refine os with arch

    Reason: On Windows

---

    Reason: On Windows i386

# skip_unless_r works as expected

    Code
      skip_unless_r("idfjdij")
    Condition
      Error in `skip_unless_r()`:
      ! `spec` must be an valid version specification, like ">= 4.0.0", not "idfjdij".

# skip_unless_r gives the expected output

    Reason: Current R version (4.5.0) does not satisfy requirement (>= 999.999.999)

---

    Reason: Current R version (4.5.0) does not satisfy requirement (== 0.0.0)

