# mock_output_sequence() works

    Code
      mocked_sequence()
    Condition
      Error in `mocked_sequence()`:
      ! Can't find value for 4th iteration.
      i `...` has only 3 values.
      i You can set `recycle` to `TRUE`.

