# now defunct

    Code
      local_mock()
    Condition
      Error:
      ! `local_mock()` was deprecated in testthat 3.2.0 and is now defunct.
      i Please use `local_mocked_bindings()` instead.
    Code
      with_mock(is_testing = function() FALSE)
    Condition
      Error:
      ! `with_mock()` was deprecated in testthat 3.2.0 and is now defunct.
      i Please use `with_mocked_bindings()` instead.

