# validates key inputs

    Code
      expect(1)
    Condition
      Error in `expect()`:
      ! `ok` must be `TRUE` or `FALSE`, not the number 1.
    Code
      expect(TRUE, 1)
    Condition
      Error in `expect()`:
      ! `failure_message` must be a character vector, not the number 1.

