% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapshot-github.R
\name{snapshot_download_gh}
\alias{snapshot_download_gh}
\title{Download snapshots from GitHub}
\usage{
snapshot_download_gh(repository, run_id, dest_dir = ".")
}
\arguments{
\item{repository}{Repository owner/name, e.g. \code{"r-lib/testthat"}.}

\item{run_id}{Run ID, e.g. \code{"47905180716"}. You can find this in the action url.}

\item{dest_dir}{Package root directory. Defaults to the current directory.}
}
\description{
If your snapshots fail on GitHub, it can be a pain to figure out exactly
why, or to incorporate them into your local package. This function makes it
easy, only requiring you to interactively select which job you want to
take the artifacts from.

Note that you should not generally need to use this function manually;
instead copy and paste from the hint emitted on GitHub. This hint is only
emitted when running in a job named "R-CMD-check", since that's where the
testthat artifact is typically uploaded.
}
