% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reporter-rstudio.R
\name{RStudioReporter}
\alias{RStudioReporter}
\title{Test reporter: RStudio}
\description{
This reporter is designed for output to RStudio. It produces results in
any easily parsed form.
}
\seealso{
Other reporters: 
\code{\link{CheckReporter}},
\code{\link{DebugReporter}},
\code{\link{FailReporter}},
\code{\link{JunitReporter}},
\code{\link{ListReporter}},
\code{\link{LocationReporter}},
\code{\link{MinimalReporter}},
\code{\link{MultiReporter}},
\code{\link{ProgressReporter}},
\code{\link{Reporter}},
\code{\link{SilentReporter}},
\code{\link{StopReporter}},
\code{\link{SummaryReporter}},
\code{\link{TapReporter}},
\code{\link{TeamcityReporter}}
}
\concept{reporters}
